// Copyright (c) 2017 M.Hirsch
// Please see LICENSE.txt for license

#include "i2c.h"

void init_i2c()
{
	bitclr(sda_port, sda_bit);
	bitclr(sda_port, scl_bit);
	sda_hi;
	scl_hi;
}

void i2c_wait_clk(void)
{
	asm volatile("nop");
}

void i2c_wait(void)
{
	for(volatile uint8_t t = i2c_wait_val; t > 0; t--);
}

void i2c_start()
{
	sda_lo;
	i2c_wait();
	scl_lo;
	i2c_wait();
}

void i2c_stop()
{
	sda_lo;
	i2c_wait();
	scl_hi;
	i2c_wait();
	sda_hi;
	i2c_wait();
}

uint8_t i2c_write(uint8_t val)
{
	uint8_t m = 0x80;
	while(m)
	{
		if(val & m)
			sda_hi;
		else
			sda_lo;
//		i2c_wait_clk();
		scl_hi;
//		i2c_wait_clk();
		m >>= 1;
		scl_lo;
	}
	m = SUCCESS;
	i2c_wait_clk();
	sda_hi;
	i2c_wait_clk();
	scl_hi;
	i2c_wait_clk();
	if(sda_tst) m = FAIL;
	scl_lo;
	i2c_wait();
	return m;
}

uint8_t i2c_read(uint8_t ack)
{
	uint8_t m = 0x80;
	uint8_t v = 0;
	sda_hi;
	while(m)
	{
		scl_hi;
		i2c_wait_clk();
		if(sda_tst) v |= m;
		scl_lo;
		i2c_wait_clk();
		m >>= 1;
	}
	if(ack) sda_lo;
	i2c_wait_clk();
	scl_hi;
	i2c_wait_clk();
	scl_lo;
	sda_hi;
	i2c_wait();
	return v;	
}

uint8_t i2c_start_adr(uint8_t adr, uint8_t rw)
{
	adr |= rw;
	i2c_start();
	return i2c_write(adr);
}
